;-------------------------------------------------------------------------------
; Script name:
;       hots
;
; Script Function:
;       Script enables use of ctags in PSPad editor. 
;
; Important globals initialized:
;   PSP_ACTION_KEY      
;   HOT_CTAGS_FILES     
;   HOT_CTAGS_FILE      
;   HOT_CASE_SENSITIVE  
;   HOT_TMP_PATH        
;   HOT_HISTORY_SIZE    
;   HOT_HISTORY_FILE    
;
;   Copyright (C) 2008, 2009 Alesc
;   This program is free software: you can redistribute it and/or modify
;   it under the terms of the GNU General Public License.
;-------------------------------------------------------------------------------

#NoEnv  ; Recommended for performance and compatibility with future AutoHotkey releases.
#WinActivateForce
SendMode Input  ; Recommended for new scripts due to its superior speed and reliability.
SetWorkingDir %A_ScriptDir%  ; Ensures a consistent starting directory.
#SingleInstance force

if InitHots() = HOT_ERR
    return

if CheckOptions() = HOT_ERR
    CreateOptionsGUI()

return
;-------------------------------------------------------------------------------
; hots   jump into function
;-------------------------------------------------------------------------------
Hot_GoTo:
DbgIn(A_ThisLabel)

selected := getSelectedTxt()
if IsValid(selected) = HOT_ERR
    return


FileRead, ctags_content, %HOT_CTAGS_FILE%
if ErrorLevel
{
    Report("Error", "Can not read ctags file[" . HOT_CTAGS_FILE . "].")
    DbgOut(A_ThisLabel)
    return
}

match_pos = 1
tag_index = 0
position = 0

loop
{
    ; ECTIOONN_DELLETE	v:\msrc\setupm\mmsii_cutil\coanfupd\reagacblean.h	22;"	d	line:22
    ; NwSmsSCheckNSInfoM	v:\msrc\ada\tneetwarem\atsaconn.c	/^NwSmsSCheckNSInfoM(selSmsResList *resource)$/;"	f	line:765
    position := RegExMatch(ctags_content, HOT_CASE_SENSITIVE_STR . "S)`r`n((" . selected . ")\t(.+)\t/?\^?(.+)\$?/?;.+line:(\d+).*)`r`n", line, match_pos)

    if (position != 0)
    {
        ; TODO find a way to remove $/ with previous regexmatch
        line4 := RegExReplace(line4, "\$/$", "")

        tag_index++
        tags_file%tag_index% = %line3%
        tags_desc%tag_index% = %line4%
        tags_line%tag_index% = %line5%
        match_pos := position + StrLen(line1)
        continue
    }
    else if (position = 0)
    {
        ; no match...end loop
        break
    }
}
ctags_content =


if (tag_index > 1)
{
    ; more mathces found...create gui and wait for user
    CreateMultitagsGUI()
}
else if (tag_index = 1)
{
    ; one match...save map file for editor to read and send shortcut
    tags_file1 := RegExReplace(tags_file1, "/", "\")
    new_line = %tags_file1% %tags_line1%

    if GetPSPCaretPos(filename, ycaret, xcaret) = HOT_ERR
    {
        DbgOut(A_ThisLabel)
        return
    }

    start_line = %filename% %ycaret%
    WriteHistLine(start_line, HOT_HISTORY_SIZE, HOT_HISTORY_FILE)
    WriteHistLine(new_line, HOT_HISTORY_SIZE, HOT_HISTORY_FILE)
    ExecPSPFunc("hots_ctag.moveto("AhkToPSPPath(HOT_HISTORY_FILE) . ")")
}
else
{
    ; selected txt not found in ctags
    Report("Warning", "Selected text [" . selected . "] not found in ctags.")
}

DbgOut(A_ThisLabel)
return
;-------------------------------------------------------------------------------
; hots   undo
;-------------------------------------------------------------------------------
Hot_Undo:
DbgIn(A_ThisLabel)

if UndoRedoHist(-1, HOT_HISTORY_SIZE, HOT_HISTORY_FILE) = HOT_ERR
{
    DbgOut(A_ThisLabel)
    return
}

ExecPSPFunc("hots_ctag.moveto("AhkToPSPPath(HOT_HISTORY_FILE) . ")")
DbgOut(A_ThisLabel)
return
;-------------------------------------------------------------------------------
; hots   redo
;-------------------------------------------------------------------------------
Hot_Redo:
DbgIn(A_ThisLabel)

if UndoRedoHist(1, HOT_HISTORY_SIZE, HOT_HISTORY_FILE) = HOT_ERR
{
    DbgOut(A_ThisLabel)
    return
}

ExecPSPFunc("hots_ctag.moveto("AhkToPSPPath(HOT_HISTORY_FILE) . ")")
DbgOut(A_ThisLabel)
return
;-------------------------------------------------------------------------------
; hots   code autocomplete
;-------------------------------------------------------------------------------
Hot_AutoComplete:
if Hot_AutoComplete_presses > 0
{
    Hot_AutoComplete_presses += 1
    return
}
Hot_AutoComplete_presses = 1
SetTimer, SomeKey, 400
return

SomeKey:
SetTimer, SomeKey, off
if Hot_AutoComplete_presses = 1 ; The key was pressed once.
{
    HotKeys("Off")
    Send, {%HOT_AUTOCOMPLETE_KEY%}
}
else if Hot_AutoComplete_presses = 2 ; The key was pressed twice.
{
    Hot_AutoComplete_presses = 0
    Goto DoubleHotKeyPress
}
else
{
    HotKeys("Off")
    loop, %Hot_AutoComplete_presses%
    {
        Send, {%HOT_AUTOCOMPLETE_KEY%}
    }
}

Hot_AutoComplete_presses = 0
HotKeys("On")
return

DoubleHotKeyPress:
DbgIn(A_ThisLabel)
DbgPrint("Double key detected [" . HOT_AUTOCOMPLETE_KEY . "].")
if (word := ExecPSPFunc("hots_ctag.hotword")) = ""
{
    DbgOut(A_ThisLabel)
    return
}

; read ctags file into memory
FileRead, ctags_content, %HOT_CTAGS_FILE%
if not ErrorLevel
{
    match_pos = 1
    position = 0
    tag_index = 0
    all_tags = 

    loop
    {
        line2 =
        position := RegExMatch(ctags_content, HOT_CASE_SENSITIVE_STR . "S)`r`n((" . word . "(\w*))\t(.+))`r`n", line, match_pos)

        if (position != 0)
        {
            ; dont write duplicates
            if InStr(all_tags, line2, true) = 0
            {
                tag_index++
                all_tags = %all_tags%%line2%
                tags%tag_index% = %line2%
                tags_piece%tag_index% = %line3%
            }
            match_pos := position + StrLen(line1)
            continue
        }
        else if (position = 0)
        {
            ; no match...end loop
            break
        }
    }
    ctags_content =

    if (tag_index > 1)
    {   ; more mathces found...create gui and wait for user
        CreateCompleteGUI()
    }
    else if (tag_index = 1)
    {   ; one match...write the remaining piece to pspad
        SendToPSP(tags_piece1, 0, 1)
    }
    else
    {   ; no match
        Report("Warning", word . " not found.")
    }
}
else
{
    Report("Error", "Ctag file[" . HOT_CTAGS_FILE . "] not found.")
}

DbgOut(A_ThisLabel)
return
;---------------------------------------
; InitHots
;
; Arguments: None
;
; Returns:
;   nothing
;
; Description:
;   Initializes menu and all global 
;   variables needed.
;---------------------------------------
InitHots()
{
    global
    local hots_icon

    TITLE = Hots
    VERSION = 0.5

    hots_icon = .\resources\hots_icon.ico
    iniHots = %A_AppData%\%Title%\%Title%.ini
    HELP_FILE = hots_help.html

    multitag_GUIhandle = 1
    completetag_GUIhandle = 1
    options_GUIhandle = 2
    about_GUIhandle = 3

    menu, ctagMenu, add

    ; custom try menu
    menu, tray, NoStandard
    menu, tray, add, Options..., menuHandler
    menu, tray, add, Change ctag, :ctagMenu
    menu, tray, add, Hint, menuHandler
    menu, tray, add, Reload, menuHandler
    menu, tray, add
    menu, tray, add, Help, menuHandler
    menu, tray, add, About..., menuHandler
    menu, tray, add
    menu, tray, add, Exit, menuHandler
    menu, tray, tip, %title% %version%
    menu, tray, icon, %hots_icon%

    ; gui list view options
    fontSize = 10
    fontType = MS sans serif
    backColor = E4E2FC

    if ((HOT_TMP_PATH := InitCORElib(title)) = HOT_ERR)
        return %HOT_ERR%

    ReadDebugEnv(title)
    DebugInit(".\log", title . "_debug.txt")
    DbgIn(A_ThisFunc)

    if ((HOT_HISTORY_FILE := InitHISTlib(HOT_TMP_PATH, "hots_ctag_history.dat")) = HOT_ERR)
    {
        DbgOut(A_ThisFunc)
        return %HOT_ERR%
    }

    LoadDefaults()

    ReadHotIni()

    UpdateCtagMenu()

    InitPSPlib(PSP_ACTION_KEY, "hots", HOT_TMP_PATH)

    Hotkeys("On")

    HINT = 
    (
    %HOT_GOTO_KEY%   jump into function
    %HOT_UNDO_KEY%   undo
    %HOT_REDO_KEY%   redo
    %HOT_AUTOCOMPLETE_KEY%   code autocomplete
    )

    DbgOut(A_ThisFunc)
    return %HOT_OK%
}
UpdateCtagMenu()
{
    global
    local n, file

    DbgIn(A_ThisFunc)

    ctagFound = 0
    menu, ctagMenu, deleteAll
    n := CntFiles(HOT_CTAGS_FILES)
    loop, %n%
    {
        file := GetFile(HOT_CTAGS_FILES, A_Index)
        menu, ctagMenu, add, %file%, ctagMenuHandler
        if HOT_CTAGS_FILE = %file%
        {
            menu, ctagMenu, ToggleCheck, %HOT_CTAGS_FILE%
            ctagFound = 1
        }
    }

    if ctagFound = 0
    {
        HOT_CTAGS_FILE =
    }

    DbgOut(A_ThisFunc)
}
LoadDefaults()
{
    global
    PSP_ACTION_KEY = ^!G
    HOT_HISTORY_SIZE = 15
    HOT_UNDO_KEY = !D
    HOT_GOTO_KEY = !R
    HOT_REDO_KEY = !G
    HOT_AUTOCOMPLETE_KEY = Ctrl
    HOT_CTAGS_FILES = c:\ctags;
    HOT_CTAGS_FILE = c:\ctags
    HOT_CASE_SENSITIVE = 1
    HOT_SEARCH_ALL = 0
    return
}
ReadHotIni()
{
    global

    DbgIn(A_ThisFunc)

    IfExist, %iniHots%
    {
        IniRead, PSP_ACTION_KEY, %iniHots%, Program options, Hots hotkey, %A_Space%
        IniRead, HOT_HISTORY_SIZE, %iniHots%, Program options, Undo buff size, %A_Space%
        IniRead, HOT_UNDO_KEY, %iniHots%, Program options, Undo, %A_Space%
        IniRead, HOT_GOTO_KEY, %iniHots%, Program options, Goto, %A_Space%
        IniRead, HOT_REDO_KEY, %iniHots%, Program options, Redo, %A_Space%
        IniRead, HOT_AUTOCOMPLETE_KEY, %iniHots%, Program options, Autocomplete, %A_Space%

        IniRead, HOT_CTAGS_FILES, %iniHots%, Ctags options, Ctags files, %A_Space%
        IniRead, HOT_CTAGS_FILE, %iniHots%, Ctags options, Ctags file, %A_Space%
        IniRead, HOT_CASE_SENSITIVE, %iniHots%, Ctags options, Case sensitive, %A_Space%
        IniRead, HOT_SEARCH_ALL, %iniHots%, Ctags options, Search all, %A_Space%

        DbgPrint("Ini file read. [" . iniHots . "]")
        DbgOut(A_ThisFunc)
        return %HOT_OK%
    }

    DbgPrint("Ini file not found [" . iniHots . "]")
    DbgOut(A_ThisFunc)
    return %HOT_ERR%
}
WriteHotIni()
{
    global

    DbgIn(A_ThisFunc)

    IniWrite, %PSP_ACTION_KEY%, %iniHots%, Program options, hots hotkey
    IniWrite, %HOT_HISTORY_SIZE%, %iniHots%, Program options, Undo buff size
    IniWrite, %HOT_UNDO_KEY%, %iniHots%, Program options, Undo
    IniWrite, %HOT_GOTO_KEY%, %iniHots%, Program options, Goto
    IniWrite, %HOT_REDO_KEY%, %iniHots%, Program options, Redo
    IniWrite, %HOT_AUTOCOMPLETE_KEY%, %iniHots%, Program options, Autocomplete

    IniWrite, %HOT_CTAGS_FILES%, %iniHots%, Ctags options, Ctags files
    IniWrite, %HOT_CTAGS_FILE%, %iniHots%, Ctags options, Ctags file
    IniWrite, %HOT_CASE_SENSITIVE%, %iniHots%, Ctags options, Case sensitive
    IniWrite, %HOT_SEARCH_ALL%, %iniHots%, Ctags options, Search all

    DbgPrint("Ini file written [" . iniHots . "]")
    DbgOut(A_ThisFunc)
    return
}
; on, off
Hotkeys(activate)
{
    global
    DbgIn(A_ThisFunc)

    if HOT_GOTO_KEY
    {
        Hotkey, %HOT_GOTO_KEY%, Hot_GoTo, %activate%
        DbgPrint(HOT_GOTO_KEY . " : " . activate)
    }
    if HOT_UNDO_KEY
    {
        Hotkey, %HOT_UNDO_KEY%, Hot_Undo, %activate%
        DbgPrint(HOT_UNDO_KEY . " : " . activate)
    }
    if HOT_REDO_KEY
    {
        Hotkey, %HOT_REDO_KEY%, Hot_Redo, %activate%
        DbgPrint(HOT_REDO_KEY . " : " . activate)
    }
    if HOT_AUTOCOMPLETE_KEY
    {
        Hotkey, %HOT_AUTOCOMPLETE_KEY%, Hot_AutoComplete, %activate%
        DbgPrint(HOT_AUTOCOMPLETE_KEY . " : " . activate)
    }
    DbgOut(A_ThisFunc)
    return
}
HelpStart()
{
    global

    DbgIn(A_ThisFunc)

    IfNotExist, %HELP_FILE%
    {
        Report("Warning", "Help file [" . HELP_FILE . "] not found.")
        DbgPrint("Help file not found. [" . HELP_FILE . "]")
        DbgOut(A_ThisFunc)
        return %HOT_ERR%
    }

    Run, %HELP_FILE%

    DbgPrint("Help file run. [" . HELP_FILE . "]")
    DbgOut(A_ThisFunc)
    return %HOT_OK%
}
menuHandler:
if (A_ThisMenuItem = "Options...")
{
    ReadHotIni()
    CreateOptionsGUI()
}
else if (A_ThisMenuItem = "Hint")
    HintDisplay(HINT)
else if (A_ThisMenuItem = "Reload")
{
    DeinitCORElib(TITLE)
    Reload
}
else if (A_ThisMenuItem = "Help")
    HelpStart()
else if (A_ThisMenuItem = "About...")
    AboutHots()
else if (A_ThisMenuItem = "Exit")
{
    DeinitCORElib(TITLE)
    ExitApp
}
return

ctagMenuHandler:
DbgIn(A_ThisLabel)

menu, ctagMenu, ToggleCheck, %A_ThisMenuItem%
HOT_CTAGS_FILE = %A_ThisMenuItem%
IniWrite, %HOT_CTAGS_FILE%, %iniHots%, Ctags options, Ctags file

n := CntFiles(HOT_CTAGS_FILES)
loop, %n%
{
    file := GetFile(HOT_CTAGS_FILES, A_Index)
    if file <> %A_ThisMenuItem%
        menu, ctagMenu, Uncheck, %file%
}
DbgOut(A_ThisLabel)
return
;-------------------------------------------------------------------------------
; Includes
;-------------------------------------------------------------------------------
#Include %A_ScriptDir%\includes\core_lib.ahk
#Include %A_ScriptDir%\includes\debug_lib.ahk
#Include %A_ScriptDir%\includes\hist_lib.ahk
#Include %A_ScriptDir%\includes\psp_lib.ahk
#Include %A_ScriptDir%\includes\hots_gui.ahk
