;-------------------------------------------------------------------------------
; Install functions
;
;   Copyright (C) 2008, 2009 Alesc
;   This program is free software: you can redistribute it and/or modify
;   it under the terms of the GNU General Public License.
;-------------------------------------------------------------------------------

#NoEnv  ; Recommended for performance and compatibility with future AutoHotkey releases.
#WinActivateForce
SendMode Input  ; Recommended for new scripts due to its superior speed and reliability.
SetWorkingDir %A_ScriptDir%  ; Ensures a consistent starting directory.
#SingleInstance force

if InitHotsSetup() = HOT_ERR
    return

CreateSetupGUI()

return
;---------------------------------------
; InitHotsSetup
;
; Arguments: None
;
; Returns:
;   nothing
;
; Description:
;   Initializes menu and all global 
;   variables needed
;
; Important globals initialized:
;   PROGRAM_INSTALL_NAME
;---------------------------------------
InitHotsSetup()
{
    global

    FIRM_NAME = Alesc Freeware
    TITLE = Hots_Setup
    PROGRAM_INSTALL_NAME = Hots
    PROG_REG_KEY = SOFTWARE\%FIRM_NAME%\%PROGRAM_INSTALL_NAME%
    VERSION = 0.5

    setup_GUIhandle = 1
    about_GUIhandle = 2

    ; custom try menu
    menu, tray, NoStandard
    menu, tray, add, Reload, menuHandler
    menu, tray, add
    menu, tray, add, About..., menuHandler
    menu, tray, add
    menu, tray, add, Exit, menuHandler
    menu, tray, tip, %TITLE% %VERSION%

    if ((SETUP_TMP_PATH := InitCORElib(TITLE)) = HOT_ERR)
        return %HOT_ERR%

    if Unpack() = HOT_ERR
        return %HOT_ERR%

    if GetInstallInfo(0) = HOT_OK
    {
        Report("Warning", PROGRAM_INSTALL_NAME . " already installed.")
        return %HOT_ERR%
    }

    return %HOT_OK%
}
;---------------------------------------
; Install
;
; Arguments: None
;
; Returns:
;   nothing
;
; Description:
;   Is called from gui when install is started.
;   Installs files to specified folder and writes
;   program registry values.
;
; Important globals initialized:
;   SETUP_ACTIONS
;---------------------------------------
Install()
{
    global

    SETUP_ACTIONS = 16
    CURRENT_ACTION = 0

    ; create dirs
    UpdateProgressText("Creating resources")
    if FileCreateDirEx(SetupPath . "\resources") = HOT_ERR
        return %HOT_ERR%
    UpdateProgressBar()

    UpdateProgressText("Creating log directory")
    if FileCreateDirEx(SetupPath . "\log") = HOT_ERR
        return %HOT_ERR%
    UpdateProgressBar()

    UpdateProgressText("Creating user's application specific directory")
    if FileCreateDirEx(A_AppData . "\" . PROGRAM_INSTALL_NAME) = HOT_ERR
        return %HOT_ERR%
    UpdateProgressBar()
    ; install pspad jscripts
    GetPSPPaths()
    UpdateProgressText("Installing hots.js")
    FileInstall, .\hots.js, %PSP_JSCRIPTS_PATH%\hots.js, 1
    UpdateProgressBar()

    UpdateProgressText("Installing hots_core.js")
    FileInstall, .\hots_core.js, %PSP_JSCRIPTS_PATH%\hots_core.js, 1
    UpdateProgressBar()

    UpdateProgressText("Installing hots_ctag.js")
    FileInstall, .\hots_ctag.js, %PSP_JSCRIPTS_PATH%\hots_ctag.js, 1
    UpdateProgressBar()
    ; install resources
    UpdateProgressText("Installing resources\hots_about.jpg")
    FileInstall, .\resources\hots_about.jpg, %SetupPath%\resources\hots_about.jpg, 1
    UpdateProgressBar()

    UpdateProgressText("Installing resources\hots_icon.ico")
    FileInstall, .\resources\hots_icon.ico, %SetupPath%\resources\hots_icon.ico, 1
    UpdateProgressBar()

    UpdateProgressText("Installing resources\hots_uninstall_icon.ico")
    FileInstall, .\resources\hots_uninstall_icon.ico, %SetupPath%\resources\hots_uninstall_icon.ico, 1
    UpdateProgressBar()
    ; install core files
    UpdateProgressText("Installing hots_help.html")
    FileInstall, .\hots_help.html, %SetupPath%\hots_help.html, 1
    UpdateProgressBar()

    UpdateProgressText("Installing COPYING.txt")
    FileInstall, .\COPYING.txt, %SetupPath%\COPYING.txt, 1
    UpdateProgressBar()

    UpdateProgressText("Installing hots.exe")
    FileInstall, .\hots.exe, %SetupPath%\hots.exe, 1
    UpdateProgressBar()

    UpdateProgressText("Installing uninstall.exe")
    FileInstall, .\uninstall.exe, %SetupPath%\uninstall.exe, 1
    UpdateProgressBar()

    ; write registry
    UpdateProgressText("Writing " . PROG_REG_KEY)
    if RegWriteEx("REG_SZ", "HKEY_CURRENT_USER", PROG_REG_KEY, "ProgramName_1", PROGRAM_INSTALL_NAME) = HOT_ERR
        return %HOT_ERR%
    UpdateProgressBar()

    UpdateProgressText("Writing " . PROG_REG_KEY)
    if RegWriteEx("REG_SZ", "HKEY_CURRENT_USER", PROG_REG_KEY, "Version", VERSION) = HOT_ERR
        return %HOT_ERR%
    UpdateProgressBar()

    UpdateProgressText("Writing " . PROG_REG_KEY)
    if RegWriteEx("REG_SZ", "HKEY_CURRENT_USER", PROG_REG_KEY, "Path", SetupPath) = HOT_ERR
        return %HOT_ERR%
    UpdateProgressBar()

    return %HOT_OK%
}

Unpack()
{
    global

    if FileCreateDirEx(SETUP_TMP_PATH . "\resources") = HOT_ERR
        return %HOT_ERR%
    FileInstall, .\resources\hots_about.jpg, %SETUP_TMP_PATH%\resources\hots_about.jpg, 1
    FileInstall, .\resources\setup_stamp.gif, %SETUP_TMP_PATH%\resources\setup_stamp.gif, 1
    FileInstall, .\resources\setup_title.gif, %SETUP_TMP_PATH%\resources\setup_title.gif, 1
}

menuHandler:
if (A_ThisMenuItem = "Reload")
{
    DeinitCORElib(TITLE)
    Reload
}
else if (A_ThisMenuItem = "About...")
{
    AboutSetup()

}
else if (A_ThisMenuItem = "Exit")
{
    DeinitCORElib(TITLE)
    ExitApp
}
return
;-------------------------------------------------------------------------------
; Includes
;-------------------------------------------------------------------------------
#Include %A_ScriptDir%\includes\core_lib.ahk
#Include %A_ScriptDir%\includes\debug_lib.ahk
#Include %A_ScriptDir%\includes\setup_core.ahk
#Include %A_ScriptDir%\includes\setup_gui.ahk
#include %A_ScriptDir%\includes\psp_lib.ahk
