;-------------------------------------------------------------------------------
; history functions
;
;   Copyright (C) 2008, 2009 Alesc
;   This program is free software: you can redistribute it and/or modify
;   it under the terms of the GNU General Public License.
;-------------------------------------------------------------------------------

InitHISTlib(hist_path, hist_file)
{
    global HOT_OK, HOT_ERR

    hist_file = %hist_path%\%hist_file%
    if FileAppendEx("", 1, hist_file) = HOT_ERR
    {
        Report("Error", "Can not initialize history lib. Invalid arguments.")
        return %HOT_ERR%
    }

    return hist_file
}
;---------------------------------------
; WriteHistLine
;
; Arguments: new_line - line to insert
;            hist_size - size of hist.buffer
;            hist_file - history file
;
; Returns:
;   HOT_OK / HOT_ERR
;
; Description:
;   Inserts new line into history file.
;---------------------------------------
WriteHistLine(new_line, hist_size, hist_file)
{
    global HOT_OK, HOT_ERR

    buffPosSign = <
    buffCnt = 0
    currBuffLine = 0

    ; measure buff and get current position
    Loop, read, %hist_file%
    {
        buffCnt += 1
        if InStr(A_LoopReadLine, buffPosSign)
            currBuffLine := buffCnt

        files%buffCnt% := RegExReplace(A_LoopReadLine, "\<", "")
    }

    ; curr.position is on top write from begining of buffer
    if (currBuffLine = hist_size)
    {
        files1 := new_line . buffPosSign
    }
    ; curr.position is in the middle of buffer
    else if (currBuffLine < hist_size)
    {
        if (buffCnt <> hist_size)
            buffCnt += 1

        currBuffLine += 1
        files%currBuffLine% := new_line . buffPosSign
    }
    else
    {
        Report("Error", "Internal error. Buffer overrun. [" . currBuffLine . " " . hist_size . "].")
        return %HOT_ERR%
    }

    ; re-write history file - ignore delete error
    FileDelete, %hist_file%
    Loop, %buffCnt%
    {
        if FileAppendEx(files%A_Index% . "`r`n", 0, hist_file) = HOT_ERR
            return %HOT_ERR%
    }

    return %HOT_OK%
}
;---------------------------------------
; UndoRedoHist
;
; Arguments: step - negative or positive number
;            hist_size - size of hist.buffer
;            hist_file - history file
;
; Returns:
;   HOT_OK, HOT_ERR
;
; Description:
;   Undo or redo in the history file. Step 
;   by any number.
;---------------------------------------
UndoRedoHist(step, hist_size, hist_file)
{
    global HOT_OK, HOT_ERR

    buffPosSign = <
    buffCnt = 0
    currBuffLine = 0

    ; measure buff and get current position
    Loop, read, %hist_file%
    {
        buffCnt += 1
        if InStr(A_LoopReadLine, buffPosSign)
            currBuffLine := buffCnt

        files%A_Index% := RegExReplace(A_LoopReadLine, "\<", "")
    }

    if buffCnt = 0
    {
        Report("Warning", "History empty or no history file.")
        return %HOT_ERR%
    }

    desiredLoc := currBuffLine + step
    ; moving forward over the top of buff
    if (desiredLoc > buffCnt)
        moveLoc := desiredLoc - buffCnt
    ; moving backward over the start of buff
    else if (desiredLoc < 1)
        moveLoc := buffCnt + desiredLoc
    ; just move will you
    else
        moveLoc := desiredLoc

    files%moveLoc% := files%moveLoc% . buffPosSign

    ; re-write history file
    FileDelete, %hist_file%
    if ErrorLevel
    {
        Report("Error", "Faild to delete history file [" . hist_file . "].")
        return %HOT_ERR%
    }
    Loop, %buffCnt%
    {
        if FileAppendEx(files%A_Index% . "`r`n", 0, hist_file) = HOT_ERR
            return %HOT_ERR%
    }

    return %HOT_OK%
}
